// Copyright  2006-2009 Travis Robinson. All rights reserved.
// 
// website: sourceforge.net/projects/libusbdotnet/
// e-mail:  libusbdotnet@gmail.com
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. or 
// visit www.gnu.org.
// 
// 
using System;
using System.Runtime.InteropServices;
using LibUsbDotNet.Internal;

#pragma warning disable 649

namespace LibUsbDotNet.Descriptors
{
    /// <summary> Usb Device Descriptor
    /// </summary> 
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public class UsbDeviceDescriptor : UsbDescriptor
    {
        /// <summary>
        /// Total size of this structure in bytes.
        /// </summary>
        public new static readonly int Size = Marshal.SizeOf(typeof (UsbDeviceDescriptor));

        /// <summary>
        /// USB Specification Number which device complies too.
        /// </summary>
        public readonly short BcdUsb;

        /// <summary>
        /// Class Code (Assigned by USB Org)
        /// If equal to Zero, each interface specifies its own class code; If equal to 0xFF, the class code is vendor specified; Otherwise field is valid Class Code.
        /// </summary>
        public readonly byte Class;

        /// <summary>
        /// Subclass Code (Assigned by USB Org)
        /// </summary>
        public readonly byte SubClass;

        /// <summary>
        /// Protocol Code (Assigned by USB Org)
        /// </summary>
        public readonly byte Protocol;

        /// <summary>
        /// Maximum Packet Size for Zero Endpoint. Valid Sizes are 8, 16, 32, 64
        /// </summary>
        public readonly byte MaxPacketSize0;

        /// <summary>
        /// Vendor ID (Assigned by USB Org)
        /// </summary>
        public readonly short VendorID;

        /// <summary>
        /// Product ID (Assigned by Manufacturer)
        /// </summary>
        public readonly short ProductID;

        /// <summary>
        /// Device Release Number
        /// </summary>
        public readonly short BcdDevice;

        /// <summary>
        /// Index of Manufacturer String Descriptor
        /// </summary>
        public readonly byte ManufacturerStringIndex;

        /// <summary>
        /// Index of Product String Descriptor
        /// </summary>
        public readonly byte ProductStringIndex;

        /// <summary>
        /// Index of Serial Number String Descriptor
        /// </summary>
        public readonly byte SerialStringIndex;

        /// <summary>
        /// Number of Possible Configurations
        /// </summary>
        public readonly byte ConfigurationCount;

        internal UsbDeviceDescriptor()
        {
        }

        ///<summary>
        ///Returns a <see cref="T:System.String"></see> that represents the current <see cref="UsbDeviceDescriptor"></see>.
        ///</summary>
        ///
        ///<returns>
        ///A <see cref="System.String"></see> that represents the current <see cref="UsbDeviceDescriptor"></see>.
        ///</returns>
        public override string ToString()
        {
            Object[] values = {
                                  Length, DescriptorType, "0x" + BcdUsb.ToString("X4"), Class, SubClass, Protocol, MaxPacketSize0,
                                  "0x" + VendorID.ToString("X4"), "0x" + ProductID.ToString("X4"), BcdDevice,
                                  ManufacturerStringIndex, ProductStringIndex, SerialStringIndex, ConfigurationCount
                              };
            string[] names = {
                                 "Length", "DescriptorType", "BcdUsb", "Class", "SubClass", "Protocol", "MaxPacketSize0", "VendorID", "ProductID",
                                 "BcdDevice",
                                 "ManufacturerStringIndex", "ProductStringIndex", "SerialStringIndex", "ConfigurationCount"
                             };

            return Helper.ToString("", names, ToStringParamValueSeperator, values, ToStringFieldSeperator);
        }
    }
}