// Copyright  2006-2009 Travis Robinson. All rights reserved.
// 
// website: sourceforge.net/projects/libusbdotnet/
// e-mail:  libusbdotnet@gmail.com
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2 of the License, or 
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. or 
// visit www.gnu.org.
// 
// 
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using LibUsbDotNet.Descriptors;
using LibUsbDotNet.Internal;

namespace LibUsbDotNet.Info
{
    /// <summary> Describes a USB device interface.
    /// </summary> 
    public class UsbInterfaceInfo : UsbBaseInfo
    {
        internal readonly UsbInterfaceDescriptor mUsbInterfaceDescriptor;
        internal List<UsbEndpointInfo> mEndpointInfo = new List<UsbEndpointInfo>();
        private String mInterfaceString;
        internal UsbDevice mUsbDevice;

        internal UsbInterfaceInfo(UsbDevice usbDevice, byte[] descriptor)
        {
            mUsbDevice = usbDevice;
            mUsbInterfaceDescriptor = new UsbInterfaceDescriptor();
            Helper.BytesToObject(descriptor, 0, Math.Min(UsbInterfaceDescriptor.Size, descriptor[0]), mUsbInterfaceDescriptor);
        }

        /// <summary>
        /// Gets the actual interface descriptor.
        /// </summary>
        public UsbInterfaceDescriptor Descriptor
        {
            get
            {
                return mUsbInterfaceDescriptor;
            }
        }

        /// <summary>
        /// Gets the collection of endpoint descriptors associated with this interface.
        /// </summary>
        public ReadOnlyCollection<UsbEndpointInfo> EndpointInfoList
        {
            get
            {
                return mEndpointInfo.AsReadOnly();
            }
        }

        /// <summary>
        /// Gets the string representation of the <see cref="UsbInterfaceDescriptor.StringIndex"/> string index.
        /// </summary>
        public String InterfaceString
        {
            get
            {
                if (ReferenceEquals(mInterfaceString, null))
                {
                    mInterfaceString = String.Empty;
                    if (Descriptor.StringIndex > 0)
                    {
                        mUsbDevice.GetString(out mInterfaceString, mUsbDevice.Info.CurrentCultureLangID, Descriptor.StringIndex);
                    }
                }
                return mInterfaceString;
            }
        }


        ///<summary>
        ///Returns a <see cref="T:System.String"></see> that represents the current <see cref="UsbInterfaceInfo"></see>.
        ///</summary>
        ///
        ///<returns>
        ///A <see cref="System.String"></see> that represents the current <see cref="UsbInterfaceInfo"></see>.
        ///</returns>
        public override string ToString()
        {
            Object[] rtn = {Descriptor.ToString(), InterfaceString};
            return String.Format("{0}InterfaceString: {1}\r\n", rtn);
        }
    }
}