﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;


using LibUsbDotNet;
using LibUsbDotNet.LibUsb;
using LibUsbDotNet.Main;
using LibUsbDotNet.Descriptors;


namespace Prox_ESC_POS_Printer
{
    /// <summary>
    /// Printer_Device 類別
    /// </summary>
    public class Printer_Device : IPrinter_Device
    {

        //SerialPort serialPort1;
        //UsbRegistry usbPort1Registry;


        public Printer_Device(SerialPort serialPort)
        {
            serialPort1 = serialPort;
        }

        public Printer_Device(UsbRegistry usbPortRegistry)
        {
            UsbPort1Registry= usbPortRegistry;
        }

        /// <summary>
        /// serial
        /// </summary>
        static SerialPort serialPort1;
        static public SerialPort Serialport
        {
            set { serialPort1 = value; }
            get { return serialPort1; }
        }

        /// <summary>
        /// usb
        /// </summary>
        /// 
        static UsbRegistry usbPort1Registry;
        static public UsbRegistry UsbPort1Registry
        {
            set { usbPort1Registry = value; }
            get { return usbPort1Registry; }

        }

        /// <summary>
        /// usb or serial 介面
        /// </summary>
        static bool usb_or_serial = true;
        static public bool Usb_or_serial
        {
            set { usb_or_serial = value; }
            get { return usb_or_serial; }
        }

        /// <summary>
        /// serial port open
        /// </summary>
        /// <param name="port_name"></param>port name
        /// <param name="brg"></param>baudrate
        /// <param name="handshake"></param>handshake
        public void Serial_Port_Open()
        {
            //serialPort1.PortName = port_name;
            //serialPort1.BaudRate = brg;
            //serialPort1.Handshake = handshake;
            serialPort1.Open();
            Init_Serial_Port();
        }

        /// <summary>
        /// Usb port open
        /// </summary>
        /// <param name="usb_port_registry"></param>UsbRegistry set
        public void Usb_Port_Open()
        {
            usbPort1Registry.Device.Open();
            Init_Local_Usb();
            Usb_or_serial = true;
            
        }


        /// <summary>
        /// UsbEndpointWriter & UsbEndpointReader
        /// </summary>
        static UsbEndpointWriter write = null;
        static public UsbEndpointWriter EndpointWriter
        {
            get
            {
                return write;
            }
            set
            {
                write = value;
            }
        }
        static UsbEndpointReader read = null;
        static public UsbEndpointReader EndpointRead
        {
            get
            {
                return read;
            }
            set
            {
                read = value;
            }
        }

        /// <summary>
        /// new  Printer  image string event
        /// </summary>
        public Printer_Image Printer_Image ;//= new Printer_Image();
        public Printer_String Printer_String; //= new Printer_String();
        public Printer_Event Printer_Event ;//= new Printer_Event();
        public Printer_Auxiliary_Functions Printer_Auxiliary; //= new Printer_Auxiliary_Functions(usbPort1Registry,write,read);
        public Printer_Barcode Printer_Barcode;


        /// <summary>
        /// Init serial
        /// </summary>
        private void Init_Serial_Port()
        {
            usb_or_serial = false;
            Printer_Event = new Printer_Event(serialPort1);
            Printer_Auxiliary = new Printer_Auxiliary_Functions(serialPort1);
            Printer_String = new Printer_String(serialPort1);
            Printer_Image = new Printer_Image(serialPort1);
            Printer_Barcode = new Printer_Barcode(serialPort1);

            serialPort1.DataReceived += new SerialDataReceivedEventHandler(Printer_Event.SerialPort1_DataReceived);

        }
        /// <summary>
        /// init usb
        /// </summary>
        private void Init_Local_Usb()
        {
            usb_or_serial = true;

            byte bCfgValue;

            //bool bRtn = Printer_Device.usbPort1Registry.Device.GetConfiguration(out bCfgValue);
            bool bRtn = usbPort1Registry.Device.GetConfiguration(out bCfgValue);
            if (bRtn)
            {
                if (bCfgValue != 1)
                {
                    usbPort1Registry.Device.SetConfiguration(1);
                }
                else
                {
                    usbPort1Registry.Device.SetConfiguration(0);
                    usbPort1Registry.Device.SetConfiguration(1);
                }

                if (usbPort1Registry.Device is LibUsbDevice) bRtn = ((LibUsbDevice)usbPort1Registry.Device).ClaimInterface(0);
            }

            write = usbPort1Registry.Device.OpenEndpointWriter(WriteEndpointID.Ep01);
            read = usbPort1Registry.Device.OpenEndpointReader(ReadEndpointID.Ep02);


            Printer_Event = new Printer_Event(usbPort1Registry, write, read);
            Printer_Auxiliary = new Printer_Auxiliary_Functions(usbPort1Registry, write, read);
            Printer_String = new Printer_String(usbPort1Registry, write, read);
            Printer_Image = new Printer_Image(usbPort1Registry, write, read);
            Printer_Barcode = new Printer_Barcode(usbPort1Registry, write, read);


            read.DataReceived += new EventHandler<EndpointDataEventArgs>(Printer_Event.USB_Read_DataReceived);
            read.DataReceivedEnabled = true;
        }
    }
}
