﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing.Text;
using System.Drawing;

using LibUsbDotNet;
using LibUsbDotNet.LibUsb;
using LibUsbDotNet.Main;
using LibUsbDotNet.Descriptors;
using System.IO.Ports;


namespace Prox_ESC_POS_Printer
{

    /// <summary>
    /// Printer_String 類別
    /// </summary>
    public class Printer_String : IPrinter_String
    {
        /// <summary>
        /// Printer_String 建構式
        /// </summary>
        Printer_Method Printer_Method_In_Printer_String;
        public Printer_String(SerialPort x)
        {
            Printer_Method_In_Printer_String = new Printer_Method(x);
        }
        public Printer_String(UsbRegistry x, UsbEndpointWriter Write, UsbEndpointReader Read)
        {
                Printer_Method_In_Printer_String = new Printer_Method((LibUsbRegistry)x,Write, Read);
        }


        /// <summary>
        /// String 初始化
        /// </summary>
        public void Printer_String_Initialize()
        {
            Printer_Status.Printer_String_Status  s = new Printer_Status.Printer_String_Status();
            Printer_String_Status_Set(s);
        }

        /// <summary>
        /// 寫入文字至暫存區 不列印
        /// </summary>
        /// <param name="txt"></param>
        public void Printer_Write(string txt)
        {
            Printer_Method_In_Printer_String.Write(txt);
        }

        /// <summary>
        /// 將暫存區文字列印 
        /// </summary>
        public void Printer_String_Print()
        {
            Printer_Method_In_Printer_String.Print_and_Feed_Forward(0);
        }

        /// <summary>
        /// 將txt文字列印
        /// </summary>
        /// <param name="txt"></param>
        public void Printer_String_Print(string txt)
        {
            Printer_Method_In_Printer_String.Write(txt);
            Printer_Method_In_Printer_String.Print_and_Feed_Forward(0);
        }

        /// <summary>
        /// Printer_String_Status 狀態設定
        /// </summary>
        /// <param name="x"></param>
        public void Printer_String_Status_Set(Printer_Status.Printer_String_Status x)
        {
            if (x.FontA)
            {
                Printer_Method_In_Printer_String.Character_Font_Select(0);
            }
            else
            {
                Printer_Method_In_Printer_String.Character_Font_Select(1);
            }

            byte low;
            byte high;
            low = Convert.ToByte(x.String_Set.X);
            high = Convert.ToByte(x.String_Set.Y);
            Printer_Method_In_Printer_String.Character_Size_Specify((int)((high << 4) | low));


            if (x.Underline)
            {
                Printer_Method_In_Printer_String.Underline_Settings(2);
            }
            else
            {
                Printer_Method_In_Printer_String.Underline_Settings(0);
            }

            if (x.Bold)
            {
                Printer_Method_In_Printer_String.Bold_Print_SpecifyCancel(1);
            }
            else
            {
                Printer_Method_In_Printer_String.Bold_Print_SpecifyCancel(0);
            }

            if (x.Double_Strike)
            {
                Printer_Method_In_Printer_String.Double_Strike_Printing_SpecifyCancel(1);
            }
            else
            {
                Printer_Method_In_Printer_String.Double_Strike_Printing_SpecifyCancel(0);
            }

            if (x.Rotate)
            {
                Printer_Method_In_Printer_String.Character_Right_Rotate_SpecifyCancel(1);
            }
            else
            {
                Printer_Method_In_Printer_String.Character_Right_Rotate_SpecifyCancel(0);
            }

            if (x.Flip)
            {
                Printer_Method_In_Printer_String.Inversion_Printing_SpecifyCancel(1);
            }
            else
            {
                Printer_Method_In_Printer_String.Inversion_Printing_SpecifyCancel(0);
            }

            if (x.Reverse)
            {
                Printer_Method_In_Printer_String.Reverse_Print_SpecifyCancel(1);
            }
            else
            {
                Printer_Method_In_Printer_String.Reverse_Print_SpecifyCancel(0);
            }

            Printer_Method_In_Printer_String.Alignment(x.String_Alignment.Get_Alignment());

            Printer_Method_In_Printer_String.Left_Margin_Set(x.Left_Margin_Set);

            Printer_Method_In_Printer_String.Character_Right_Space_Amount_Set(x.Space);

            Printer_Method_In_Printer_String.Line_Spacing_Set(x.Up_Down_Dots);

            Printer_Method_In_Printer_String.Print_Density_Specify(x.Density);

        }

        /// <summary>
        /// 依 Status 狀態設定 列印文字
        /// </summary>
        /// <param name="x"></param>
        /// <param name="txt"></param>
        public void Printer_String_Print(Printer_Status.Printer_String_Status x, string txt)
        {
            Printer_String_Status_Set(x);

            Printer_Method_In_Printer_String.Write(txt);
            Printer_Method_In_Printer_String.Print_and_Feed_Forward(0);
            
        }


        
        public void Printer_Special_String_Print(Printer_Status.Printer_String_Status x,Font font,string txt)
        {
            Printer_String_Status_Set(x);

            Printer_Method_In_Printer_String.Downloaded_Character_Registration(3,txt,font);

            Printer_Method_In_Printer_String.Downloaded_Character_Set_SpecifyCancel(1);

            byte[] txt_string=new byte[txt.Length];
            for (int i = 0; i < txt.Length; i++)
            {
                txt_string[i] = Convert.ToByte(i + Convert.ToInt16(0x20));
            }
            Printer_Method_In_Printer_String.Write(txt_string);
            Printer_Method_In_Printer_String.Downloaded_Character_Set_SpecifyCancel(0);
            Printer_Method_In_Printer_String.Print_and_n_Lines_Feed_Forward(0);

        }


    }
}
